/* Appearance.m
   
   Appearance preferences module
    
   Created by Stefan Urbanek on 1999 Nov 1
*/

#import "Appearance.h"
#import <AppKit/NSBox.h>
#import <AppKit/NSPopUpButton.h>
#import <Foundation/NSUserDefaults.h>
#import <Foundation/NSEnumerator.h>
#import <Foundation/NSDictionary.h>

@implementation NSArray(DictionaryLookup)
- dictWithKey:(NSString *)key equalTo:(NSString *)str
{
    NSEnumerator *enumerator = [self objectEnumerator]; 
    NSDictionary *dict;

    while (dict = [enumerator nextObject]) 
    {
        if([[dict objectForKey:key] isEqual:str])
            return dict;
    }   
    return nil;
}
@end

@interface Appearance(Private)
- (void)createView;
@end

@implementation Appearance
- init
{
    NSDictionary *specDict;
    NSString     *specFileName;

    [super init];
    if(!view)
        [self createView];
        
    defaults = [NSUserDefaults standardUserDefaults];
    specFileName = [[NSBundle bundleForClass:[self class]] 
                              pathForResource: @"GNUstepSpec"
                              ofType: @"plist" ];

    specDict = [NSDictionary dictionaryWithContentsOfFile:specFileName];
    
    ifsNames = [[specDict objectForKey:@"Interface Styles"] retain];
    
    [self createView];
    [self revert:self];

    return self;
}

- (void)revert:sender
{
    NSString *str;
    NSDictionary *dict;

    [defaults synchronize]; 

    /* Revert Interface Style */
    str = [[defaults persistentDomainForName:@"NSGlobalDomain"] 
                     objectForKey:@"NSInterfaceStyle"];
    dict = [ifsNames dictWithKey:@"Name" equalTo:str];
    if(dict)
        [ifaceStyles selectItemWithTitle:[dict objectForKey:@"HumanReadableName"]];
}

- (void)save:sender
{
    NSDictionary *dict;
    NSMutableDictionary *pd;
    NSString *str;

    /* FIXME: is this safe ? 
        create somthing like
    */
    pd = [NSMutableDictionary dictionaryWithDictionary:
                        [defaults persistentDomainForName:@"NSGlobalDomain"]];

    /* Save Interface Style */
    str = [ifaceStyles titleOfSelectedItem];
    dict = [ifsNames dictWithKey:@"HumanReadableName" equalTo:str];
    if(dict)
    {
        [pd setObject:[dict objectForKey:@"Name"] forKey:@"NSInterfaceStyle"];
        [defaults setPersistentDomain:pd forName:@"NSGlobalDomain"];
    }
    [defaults synchronize]; 
}

- (void)dealloc
{
    [ifaceStyles release];
    [ifsNames release];
    [super dealloc];
}

- (void)setInterfaceStyle:(id)sender
{ 
    /* nothing yet */
}

- (void)createView
{
    NSRect viewRect   = PreferencesLayoutViewFrame;
    NSRect buttonRect = {{4.0,4.0},{128.0,20.0}};
    NSRect ifsBoxRect = {{10.0,150.0},{148.0,64.0}};
    NSBox  *box;
    NSEnumerator *enumerator = [ifsNames objectEnumerator]; 
    id            object;
    
    view = [[[NSView alloc] initWithFrame:viewRect] autorelease];
    ifaceStyles = [[NSPopUpButton alloc] initWithFrame:buttonRect];
    while (object = [enumerator nextObject]) 
    {
        [ifaceStyles addItemWithTitle:[object objectForKey:@"HumanReadableName"]];
    }
    [ifaceStyles setTarget:self];
    [ifaceStyles setAction:@selector(setInterfaceStyle:)];
    box = [[[NSBox alloc] initWithFrame:ifsBoxRect] autorelease];
    [box setTitle:@"Interface Style"];
    [box setBorderType: NSGrooveBorder];

    [box addSubview:ifaceStyles];
    [view addSubview:box];
}
@end
