/*  IconView.m 

    Created by Stefan Urbanek on oct 20 1999
*/

#import "IconView.h"
#import "Preferences.h"
#import "PreferencesInfo.h"
#import <AppKit/NSView.h>
#import <AppKit/NSImage.h>
#import <AppKit/NSMatrix.h>
#import <AppKit/NSScrollView.h>
#import <AppKit/NSScroller.h>
#import <AppKit/NSButtonCell.h>
#import <Foundation/NSString.h>

@interface NSMatrix(_RepresentedObjectSearching)
- (NSCell *)cellWithRepresentedObject:object;
@end

@implementation NSMatrix(_RepresentedObjectSearching)
- (NSCell *)cellWithRepresentedObject:object
{
    NSEnumerator *enumerator = [[self cells] objectEnumerator];
    NSCell *myCell;

/*  

    FIXME: NSMatrix -cells does not work properely
    
    while(myCell = [enumerator nextObject])
    {
        if([myCell representedObject] == object)
            return myCell;
    }
*/
    return nil;
}
@end

@implementation IconView
- initWithFrame:(NSRect) rect
{
    NSButtonCell *cellPrototype;
    NSRect scrollRect = NSMakeRect(0.0, 0.0, NSWidth(rect)-1, NSHeight(rect)-1);
    
    cellPrototype = [NSButtonCell new] ;
    [cellPrototype setButtonType:NSOnOffButton];
    [cellPrototype setImagePosition:NSImageOnly];
    [cellPrototype setAction:@selector(switchPreferences:)];
    [cellPrototype setTarget:self];
    
    [super initWithFrame:rect];
    scrollView = [[NSScrollView alloc] initWithFrame: scrollRect];

    matrix = [NSMatrix alloc];
    [matrix initWithFrame:NSZeroRect
            mode:NSRadioModeMatrix
            prototype:cellPrototype
            numberOfRows:0
            numberOfColumns:0];

    [matrix setCellSize:NSMakeSize(64.0,64.0)];

    [scrollView setDocumentView:matrix];
    [scrollView setHasHorizontalScroller:YES];
    [scrollView setBorderType:NSBezelBorder];
    [[scrollView horizontalScroller] setArrowsPosition:NSScrollerArrowsNone];
    [self addSubview:scrollView];

    preferences = [Preferences sharedPreferences];
}

- (void) addPreferences:(PreferencesInfo *)prefs
{
    NSButtonCell *cell;

    [matrix addColumn];

    cell = [matrix cellAtRow:0 column:[matrix numberOfColumns] - 1];

    [cell setImage:[prefs icon]];
    [cell setRepresentedObject:prefs];

    [matrix sizeToCells];
    [matrix setNeedsDisplay: YES];
}

/* this will be used for disabling preferences */
-(void) removePreferences:(PreferencesInfo *)prefs
{
    NSCell *cell = [matrix cellWithRepresentedObject:prefs];
    if(cell)
    {
        int i;
        int n = [matrix numberOfColumns];
        
        for(i=0;i<n;i++)
        {
            if([matrix cellAtRow:0 column:i] == cell)
            {
                [matrix removeColumn:i];
                break;
            }
        }
        [matrix sizeToCells];
        [matrix setNeedsDisplay: YES];
    }
}

-(void)selectPreferences:(PreferencesInfo *)prefs
{
    NSCell *cell = [matrix cellWithRepresentedObject:prefs];
    if(cell)
    {
        int i;
        int n = [matrix numberOfColumns];
        
        for(i=0;i<n;i++)
        {
            if([matrix cellAtRow:0 column:i] == cell)
            {
                [matrix selectCellAtRow:0 column:i];
                [matrix scrollCellToVisibleAtRow:0 column:i];
                /* FIXME: is this necessary after selectCellAtRow:column ?*/
                [self switchPreferences:self];
                break;
            }
        }
        [matrix sizeToCells];
        [matrix setNeedsDisplay: YES];
    }
}

-(void) dealloc
{
    [matrix release];
    [scrollView release];
    [super dealloc];
}
- (PreferencesInfo *)selectedPreferences
{
    return [[matrix selectedCell] representedObject];
}

-(void)switchPreferences:sender
{
    NSCell *cell = [matrix selectedCell];

    /* FIXME: how is this meant to work (I mead all that did*, will* stuff ? */
    [preferences preferencesViewWillSwitchTo:[cell representedObject]];
    [preferences preferencesViewDidSwitchTo:[cell representedObject]];
}
/*
- (void) setModifiedFlag:(BOOL)flag forPreferences:(PreferencesInfo *)prefs;
*/
@end
