/* Preferences.m

   Preferences.app Controller
    
   Created by Stefan Urbanek on Oct 20 1999
*/

#import "Preferences.h"
#import "IconView.h"
#import "PreferencesInfo.h"
#import "PreferencesLayout.h"
#import <AppKit/NSBox.h>
#import <AppKit/NSButton.h>
#import <AppKit/NSImage.h>
#import <AppKit/NSScrollView.h>
#import <AppKit/NSWindow.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSBundle.h>
#import <Foundation/NSEnumerator.h>
#import <Foundation/NSException.h>
#import <Foundation/NSFileManager.h>
#import <Foundation/NSProcessInfo.h>
#import <Foundation/NSString.h>

@class NSApplication;

static Preferences *PreferencesSharedInstance = nil;
NSRect PreferencesLayoutViewFrame = {{  0.0, 0.0},{520.0,235.0}};

static NSString *PreferencesFileExtension = @"gpreferences";

static NSBundle *PrefsMainBundle;

@interface NSMutableArray(UnicateObjectAdding)
- addUnicateObject:anObject;
@end

@implementation NSMutableArray(UnicateObjectAdding)
- addUnicateObject:anObject
{
    if([self containsObject:anObject] == NO)
        [self addObject:anObject];
    return self;
}
@end

static NSString *localize(NSString *str)
{
    /* FIXME: make it work */
    /*
    return [PrefsMainBundle localizedStringForKey:str value:nil table:nil];
    */
    
    return str;
}


@implementation Preferences
+ sharedPreferences
{
    if(!PreferencesSharedInstance)
    {
        PreferencesSharedInstance = [[Preferences alloc] init];
        PrefsMainBundle = [NSBundle mainBundle];
    }
    return PreferencesSharedInstance;
}

- (void)dealloc
{
    [preferences release];
    [window release];
    [super dealloc];
}

- (void)initUI
{
    NSRect windowRect = {{100.0,200.0},{520.0,345.0}};
    NSRect layoutRect = PreferencesLayoutViewFrame;
    NSRect iconsRect  = {{  8.0,250.0},{504.0, 88.0}};
    NSRect lineRect        = {{ -1.0,240.0},{521.0,  2.0}};
    NSBox *box;
    
    window = [NSWindow alloc];
    [window initWithContentRect:windowRect
            styleMask:NSTitledWindowMask
            backing:NSBackingStoreRetained
            defer:YES];

    [window setTitle:localize(@"Preferences")];

    iconView = [IconView alloc];
    [[iconView initWithFrame:iconsRect] autorelease];
    [[window contentView] addSubview:iconView];

    layoutView = [[[NSButton alloc] initWithFrame:layoutRect] autorelease];
    [layoutView setTitle:localize(@"(No preferences loaded)")];
    [layoutView setEnabled:NO];
    [[window contentView] addSubview:layoutView];

    box = [[[NSBox alloc] initWithFrame:lineRect] autorelease];
    [box setBorderType:NSGrooveBorder];
    [box setTitlePosition:NSNoTitle];
    [[window contentView] addSubview:box];
}

- (void)showPreferences:(PreferencesInfo *)prefs
{
    [iconView selectPreferences:prefs];
}

- (void)showPreferencesNamed:(NSString *)name
{
    [self showPreferences:[self preferencesWithName:name]];
}

/* --- Delegate Methods ----------------------------------------------- */
- (void)applicationDidFinishLaunching:notification
{
    [self initUI];

    [self loadPreferencesFromPath:[[NSBundle mainBundle] bundlePath]];
    [self loadPreferencesFromKnownDirectories];

    [self showPreferences:[preferences objectAtIndex:0]];
    NSLog(localize(@"%i preferences loaded"),[preferences count]);

    [window makeKeyAndOrderFront:nil];
}

- (PreferencesInfo *)selectedPreferences
{
    return [iconView selectedPreferences];
}

- (PreferencesLayout *)selectedLayout
{
    return [[self selectedPreferences] layout];
}

/* --- Target Methods ------------------------------------------------- */

- (void)savePreferences:sender
{
    [[self selectedLayout] save:self];
}

- (void)revertPreferences:sender
{
    [[self selectedLayout] revert:self];
    [layoutView setNeedsDisplay:YES];
}

- (void)saveAllPreferences:sender
{
    NSEnumerator *enumerator = [preferences objectEnumerator]; 
    PreferencesInfo *prefs; 
    while (prefs = [enumerator nextObject]) 
        [[prefs layout] save:self];
}

- (void)revertAllPreferences:sender
{
    NSEnumerator *enumerator = [preferences objectEnumerator]; 
    PreferencesInfo *prefs; 
    
    while (prefs = [enumerator nextObject]) 
        [[prefs layout] revert:self];

    [layoutView setNeedsDisplay:YES];
}

- (void) preferencesViewWillSwitchTo:(PreferencesInfo *)prefs
{
    NSView *newView;
    PreferencesInfo *currentPrefs = [self selectedPreferences]; 
    PreferencesLayout *layout = [self selectedLayout];
    
    [layout willUnselect:self];

    newView = [[prefs layout] view];

    [self updatePreferencesWindowTitle:self];

    if(newView){
        id temp;
        [[window contentView] replaceSubview:layoutView with:newView];
        layoutView = newView;
    }
    else
        NSLog(localize(@"No view for module '%@'"), [prefs preferencesName]);

    [layout didUnselect:self];
    [[prefs layout] willSelect:self];
    [layoutView setNeedsDisplay:YES];
}

- (void) preferencesViewDidSwitchTo:(PreferencesInfo *)module
{
    [[self selectedLayout] didSelect:self];
}

- (void)updatePreferencesWindowTitle:sender
{
    [window setTitle:[[self selectedPreferences] longName]];
}

/* --- Preferences moudule loading ------------------------------------ */
- (void)loadPreferencesFromKnownDirectories
{
    NSDictionary *env = [[NSProcessInfo processInfo] environment];
    NSEnumerator *enumerator;
    NSMutableArray *roots = [[[NSMutableArray alloc] init] autorelease];
    NSString *path;
    NSString *str;

    /* NOTE: roots is NSArray insetead of NSSet to preserve directory order */
    
    /* this is modified code from NSBundle.m */
    
    str = [env objectForKey: @"GNUSTEP_USER_ROOT"];
    if(str == nil)
        str = [NSString stringWithFormat: @"%@/GNUstep", NSHomeDirectory()];
    [roots addUnicateObject:str];

    str = [env objectForKey: @"GNUSTEP_LOCAL_ROOT"];
    if (str == nil)
        str = @"/usr/GNUstep/Local";
    [roots addUnicateObject: str];

    str = [env objectForKey: @"GNUSTEP_SYSTEM_ROOT"];
    if (str == nil)
        str = @"/usr/GNUstep";
    [roots addUnicateObject: str];

    str = [env objectForKey: @"GNUSTEP_PATHPREFIX_LIST"];
    if((str != nil) && ([str isEqualToString:@""] == NO))
    {
        NSArray *paths = [str componentsSeparatedByString:@":"];
        enumerator = [paths objectEnumerator];

        while( (path = [enumerator nextObject]) )
        {
            if([path isEqualToString:@""] == NO)
                [roots addUnicateObject:path];
        }
    }
    
    enumerator = [roots objectEnumerator];
    while( (path = [enumerator nextObject]) )
    {
        [self loadPreferencesFromPath:
                  [path stringByAppendingPathComponent:@"Library/Preferences"]];
    }
}

- (void)loadPreferencesFromPath:(NSString *)aPath
{
    NSDirectoryEnumerator *direnum;
    NSString *fname;
    NSString *type;
    BOOL      flag,isDir;
/*
    flag = [[NSFileManager defaultManager] fleExistsAtPath:aPath 
                                              isDirectory:&isDir];
    if(flag == NO || isDir == NO)
        return;         
*/
   /* Just ignore it, do not say anything */
    
    direnum = [[NSDirectoryEnumerator alloc]
                    initWithDirectoryPath:aPath 
                    recurseIntoSubdirectories:NO
                    followSymlinks:NO
                    prefixFiles:NO];

    while (fname = [direnum nextObject]) { 
        type = [[direnum fileAttributes] objectForKey:NSFileType];
        if ([[fname pathExtension] isEqualToString:PreferencesFileExtension]
             && [type isEqualToString:NSFileTypeDirectory]) 
        { 
            [self loadPreferences:[aPath stringByAppendingPathComponent:fname]];
        } 
    }
}

- (PreferencesInfo *)loadPreferences:(NSString *)aPath
{
    NSBundle *bundle;
    NSString *name;
    NSString *longName;
    NSString *str;
    NSImage  *icon;
    NSDictionary *info;
    PreferencesInfo *prefs;
    Class    class;

    name = [[aPath stringByDeletingPathExtension] lastPathComponent];   
    if([self preferencesWithName:name])
    {
        NSLog(localize(@"Warning: preferences '%@' already loaded (%@)"), name, aPath);
        return [self preferencesWithName:name];
    }

    bundle = [NSBundle bundleWithPath:aPath];
    if(!bundle)
    {
        NSLog(localize(@"Error loading bundle for preferences '%@'"), aPath);
        return nil;
    }

    NSLog(localize(@"Loading preferences %@..."),name);
    info = [[bundle infoDictionary] objectForKey:@"PreferencesInfo"];
    if(!info)
        NSLog(@"Unable to read info for '%@' (ignored)",name);
    
    /* Run test program and decide whether to continue or not */
    str = [info objectForKey:@"TestProgram"];

    if((str != nil) && ([str isEqualToString:@""] == NO))
    {
        NSString *prg;
        NSLog(localize(@"Running test program for '%@'"),name);
        prg = [aPath stringByAppendingPathComponent: [NSBundle _gnustep_target_cpu]];
        prg = [prg stringByAppendingPathComponent: [NSBundle _gnustep_target_os]];
        prg = [prg stringByAppendingPathComponent: [NSBundle _library_combo]];
        prg = [prg stringByAppendingPathComponent:str];
        
        if([self runTestProgram:prg] == NO)
        {
            NSLog(localize(@"Test program failed, '%@' not loaded"),name);
            return nil;
        }
    }
    /* Extract Preferences name from path */
    longName = localize([info objectForKey:@"LongName"]);
    if((longName == nil) || ([longName isEqualToString:@""] == YES))
        longName = [name stringByAppendingString:localize(@" Preferences")];

    /* Load icon */
    str = [bundle pathForResource:name ofType:@"tiff"];

    icon = [[[NSImage alloc] initWithContentsOfFile:str] autorelease];
    if(!icon)
        /* FIXME: use some default icon */
        NSLog(localize(@"Warning: no icon"));

    /* FIXME: just store information here, budle will be loaded later*/
    class = [bundle principalClass];

    prefs = [PreferencesInfo alloc];
    [prefs initWithLayout:[[[class alloc] init] autorelease]
                     icon:icon 
                     name:name
                 longName:longName];
    
    [prefs autorelease]; /* they will be owned by preferences array */
    [self registerPreferences:prefs];

    return prefs;
}

- (void) registerPreferences:(PreferencesInfo *)prefs
{
    if(!preferences)
        preferences = [[NSMutableArray alloc] init];
    [preferences addObject:prefs];
    [iconView addPreferences:prefs];
}

- (BOOL) runTestProgram:(NSString *)aPath
{
    NSTask *program;
    BOOL   fail = NO;
 
    NS_DURING
        program = [NSTask launchedTaskWithLaunchPath:aPath arguments:nil];
    NS_HANDLER 
        if ([[localException name] isEqualToString:NSInvalidArgumentException]) 
        { 
            NSLog(localize(@"Unable to run test program '%@'"),aPath);
            /* FIXME: temporary hack */
            fail = YES;
            /*
            NS_VALRETURN(NO);
            */
        } 
        else
            [localException raise]; 
    NS_ENDHANDLER    
    
    if(fail)
        return NO;
        
    [program waitUntilExit]; 
    return ([program terminationStatus] == 0); 
}

- (PreferencesInfo *) preferencesWithName:(NSString *)name;
{
    NSEnumerator *e = [preferences objectEnumerator];
    PreferencesInfo *prefs;
    
    while( (prefs = [e nextObject]) )
    {
        if([[prefs preferencesName] isEqualToString:name] == YES)
            return prefs;
    }
    return nil;
}

-(BOOL)application:(NSApplication *)app openFile:(NSString *)filename
{
    PreferencesInfo *prefs = [self loadPreferences:filename];
    if( prefs == nil);
        return NO;

    [self showPreferences:prefs];

    return YES;
}
@end
